(define (domain beluga)
   (:requirements :strips :typing)
   (:types
      part rack num num-swaps part-type production-line - object
   )
   (:constants
        part1 part2 part3 part4 - part
        start end none - part
        n00 - num
        plA plB - production-line
   )

   (:predicates
      (ready-to-unload ?p - part) ; 
      (on-rack ?p - part) ; 
      (left ?r - rack ?p - part) ; 
      (right ?r - rack ?p - part) ; 
      (size ?p - part ?n - num) ; 
      (level ?r - rack ?n - num) ; 
      (fit-n-sum ?n1 ?n2 ?nSum - num) ; 
      (rack-succ ?p1 ?p2 - part) ;

      (p-line-next ?pl - production-line ?p - part) ;
      (p-line-succ ?p1 ?p2 - part) ;
      (needed-next ?p - part) ;

      (processed ?p - part) ; 
      (unused ?r - rack) ; 
   )

   (:functions
      (total-cost) - number
   )

   ;; actions to switch planes
   ;; - all parts from the old plane needs to be stored
   ;; - then the parts from the new one can be stored

   (:action next-plane-1
      :parameters ()
      :precondition (and
         (on-rack part1)
         (on-rack part2)
         (on-rack part3)
      )
      :effect (and
         (ready-to-unload part4)
      )
   )

   
   ;; unload part from plane and put it on rack

   (:action put-on-rack
      :parameters (?p - part ?r - rack ?pFirst - part ?pSize ?rLevel ?newLevel - num)
      :precondition (and
         (not (= ?p ?pFirst))
         (ready-to-unload ?p)
         (left ?r ?pFirst)

         (size ?p ?pSize)
         (level ?r ?rLevel)
         (fit-n-sum ?rLevel ?pSize ?newLevel)
      )
      :effect (and
         (on-rack ?p)
         (not (left ?r ?pFirst))
         (left ?r ?p)
         (rack-succ ?p ?pFirst)
         (not (ready-to-unload ?p))
         (not (level ?r ?rLevel))
         (level ?r ?newLevel)
      )
   )


   (:action put-on-rack-first
      :parameters (?p - part ?r - rack ?pSize - num)
      :precondition (and
         (level ?r n00)
         (ready-to-unload ?p)

         (size ?p ?pSize)
         (fit-n-sum n00 ?pSize ?pSize)
      )
      :effect (and
         (on-rack ?p)
         (not (unused ?r))
         (left ?r ?p)
         (right ?r ?p)
         (not (ready-to-unload ?p))
         (not (level ?r n00))
         (level ?r ?pSize)
      )
   )


   (:action set-next
      :parameters (?pl - production-line ?p0 ?pn0 ?p1 ?pn1 - part)
      :precondition (and
         (needed-next none)
         (p-line-next plA ?p0)
         (p-line-succ ?p0 ?pn0)
         (p-line-next plB ?p1)
         (p-line-succ ?p1 ?pn1)
      )
      :effect (and
         (not (needed-next none))
         (probabilistic
            1/2 (and (needed-next ?p0) (not (p-line-next plA ?p0)) (p-line-next plA ?pn0))
            1/2 (and (needed-next ?p1) (not (p-line-next plB ?p1)) (p-line-next plB ?pn1))
         )
      )
   )


   (:action to-fab
      :parameters (?r - rack ?pRight ?pBefore ?pNeededAfterw - part ?pSize ?rLevel ?newLevel - num)
      :precondition (and
         (not (= ?pRight ?pBefore))
         (not (= ?pRight ?pNeededAfterw))
         (needed-next ?pRight) ;; needed next according to working plan

         (right ?r ?pRight)
         (rack-succ ?pBefore ?pRight)

         (size ?pRight ?pSize)
         (level ?r ?rLevel)
         (fit-n-sum ?newLevel ?pSize ?rLevel)
      )
      :effect (and
         (not (right ?r ?pRight)) ;; maintain rack position
         (not (rack-succ ?pBefore ?pRight))
         (right ?r ?pBefore)

         (not (needed-next ?pRight))
         (needed-next none)

         (not (on-rack ?pRight))
         (processed ?pRight)

         (not (level ?r ?rLevel))
         (level ?r ?newLevel)
      )
   )


   (:action to-fab-last
      :parameters (?r - rack ?p ?pNeededAfterw - part ?pSize ?rLevel ?newLevel - num)
      :precondition (and
         (needed-next ?p) ;; needed next according to working plan

         (right ?r ?p)
         (left ?r ?p)

         (size ?p ?pSize)
         (level ?r ?pSize) ;; this ensures that it is the last element
      )
      :effect (and
         (not (right ?r ?p))
         (not (left ?r ?p))

         (not (needed-next ?p))
         (needed-next none)

         (not (on-rack ?p))
         (processed ?p)

         (not (level ?r ?pSize))
         (level ?r n00)
      )
   )


   (:action change-rack-right
      :parameters (?p ?pBefore ?pNewPred - part ?rFrom ?rTo - rack ?pSize ?rFromLevel ?rFromLevelNew ?rToLevel ?rToLevelNew - num)
      :precondition (and
         (not (= ?rFrom ?rTo))
         (not (= ?p ?pBefore))
         (not (= ?p ?pNewPred))
         (not (= ?pBefore ?pNewPred))
         (right ?rFrom ?p)
         (rack-succ ?pBefore ?p)
         (right ?rTo ?pNewPred)

         (size ?p ?pSize)
         (level ?rFrom ?rFromLevel)
         (level ?rTo ?rToLevel)
         (fit-n-sum ?rFromLevelNew ?pSize ?rFromLevel)
         (fit-n-sum ?rToLevel ?pSize ?rToLevelNew)
      )
      :effect (and
         (not (right ?rFrom ?p))
         (not (rack-succ ?pBefore ?p))
         (not (right ?rTo ?pNewPred))
         (right ?rFrom ?pBefore)
         (rack-succ ?pNewPred ?p)
         (right ?rTo ?p)

         (not (level ?rFrom ?rFromLevel))
         (not (level ?rTo ?rToLevel))
         (level ?rFrom ?rFromLevelNew)
         (level ?rTo ?rToLevelNew)
         (increase (total-cost) 1)
      )
   )


   ;; the target rack is empty
   (:action change-rack-right-first
      :parameters (?p ?pBefore - part ?rFrom ?rTo - rack ?pSize ?rFromLevel ?rFromLevelNew - num)
      :precondition (and
         (not (= ?rFrom ?rTo))
         (not (= ?p ?pBefore))
         (right ?rFrom ?p)
         (rack-succ ?pBefore ?p)
         (level ?rTo n00)

         (size ?p ?pSize)
         (level ?rFrom ?rFromLevel)
         (fit-n-sum ?rFromLevelNew ?pSize ?rFromLevel)
         (fit-n-sum n00 ?pSize ?pSize)
      )
      :effect (and
         (not (right ?rFrom ?p))
         (not (unused ?rTo))
         (not (rack-succ ?pBefore ?p))
         (right ?rFrom ?pBefore)
         (left ?rTo ?p)
         (right ?rTo ?p)

         (not (level ?rFrom ?rFromLevel))
         (not (level ?rTo n00))
         (level ?rFrom ?rFromLevelNew)
         (level ?rTo ?pSize)
         (increase (total-cost) 1)
      )
   )


   (:action change-rack-right-last
      :parameters (?p ?pNewPred - part ?rFrom ?rTo - rack ?pSize ?rToLevel ?rToLevelNew - num)
      :precondition (and
         (not (= ?rFrom ?rTo))
         (not (= ?p ?pNewPred))

         (size ?p ?pSize)

         (right ?rFrom ?p)
         (left ?rFrom ?p)
         (level ?rFrom ?pSize)

         (right ?rTo ?pNewPred)

         (level ?rTo ?rToLevel)
         (fit-n-sum ?rToLevel ?pSize ?rToLevelNew)
      )
      :effect (and
         (not (right ?rFrom ?p))
         (not (left ?rFrom ?p))
         (not (right ?rTo ?pNewPred))
         (rack-succ ?pNewPred ?p)
         (right ?rTo ?p)

         (not (level ?rFrom ?pSize))
         (not (level ?rTo ?rToLevel))
         (level ?rFrom n00)
         (level ?rTo ?rToLevelNew)
         (increase (total-cost) 1)
      )
   )


   ;; this is symmetric, there is no left/right
   (:action change-rack-first-last
      :parameters (?p - part ?rFrom ?rTo - rack ?pSize - num)
      :precondition (and
         (not (= ?rFrom ?rTo))

         (size ?p ?pSize)
         (right ?rFrom ?p)
         (left ?rFrom ?p)
         (level ?rFrom ?pSize)
         (level ?rTo n00)
         (fit-n-sum n00 ?pSize ?pSize)
      )
      :effect (and
         (not (right ?rFrom ?p))
         (not (left ?rFrom ?p))
         (right ?rTo ?p)
         (not (unused ?rTo))

         (not (level ?rFrom ?pSize))
         (not (level ?rTo n00))
         (level ?rFrom n00)
         (level ?rTo ?pSize)
         (increase (total-cost) 1)
      )
   )


   (:action change-rack-left
      :parameters (?p ?pAfter ?pNewSucc - part ?rFrom ?rTo - rack ?pSize ?rFromLevel ?rFromLevelNew ?rToLevel ?rToLevelNew - num)
      :precondition (and
         (not (= ?rFrom ?rTo))
         (not (= ?p ?pAfter))
         (not (= ?p ?pNewSucc))
         (not (= ?pAfter ?pNewSucc))
         (left ?rFrom ?p)
         (rack-succ ?p ?pAfter)
         (left ?rTo ?pNewSucc)

         (size ?p ?pSize)
         (level ?rFrom ?rFromLevel)
         (level ?rTo ?rToLevel)
         (fit-n-sum ?rFromLevelNew ?pSize ?rFromLevel)
         (fit-n-sum ?rToLevel ?pSize ?rToLevelNew)
      )
      :effect (and
         (not (left ?rFrom ?p))
         (not (rack-succ ?p ?pAfter))
         (not (left ?rTo ?pNewSucc))
         (left ?rFrom ?pAfter)
         (rack-succ ?p ?pNewSucc)
         (left ?rTo ?p)

         (not (level ?rFrom ?rFromLevel))
         (not (level ?rTo ?rToLevel))
         (level ?rFrom ?rFromLevelNew)
         (level ?rTo ?rToLevelNew)
         (increase (total-cost) 1)
      )
   )


   ;; the target rack is empty
   (:action change-rack-left-first
      :parameters (?p ?pAfter - part ?rFrom ?rTo - rack ?pSize ?rFromLevel ?rFromLevelNew - num)
      :precondition (and
         (not (= ?rFrom ?rTo))
         (not (= ?p ?pAfter))
         (left ?rFrom ?p)
         (rack-succ ?p ?pAfter)
         (level ?rTo n00)

         (size ?p ?pSize)
         (level ?rFrom ?rFromLevel)
         (fit-n-sum ?rFromLevelNew ?pSize ?rFromLevel)
         (fit-n-sum n00 ?pSize ?pSize)
      )
      :effect (and
         (not (left ?rFrom ?p))
         (not (rack-succ ?p ?pAfter))
         (left ?rFrom ?pAfter)
         (left ?rTo ?p)
         (right ?rTo ?p)
         (not (unused ?rTo))

         (not (level ?rFrom ?rFromLevel))
         (not (level ?rTo n00))
         (level ?rFrom ?rFromLevelNew)
         (level ?rTo ?pSize)
         (increase (total-cost) 1)
      )
   )


   (:action change-rack-left-last
      :parameters (?p ?pNewSucc - part ?rFrom ?rTo - rack ?pSize ?rToLevel ?rToLevelNew - num)
      :precondition (and
         (not (= ?rFrom ?rTo))
         (not (= ?p ?pNewSucc))

         (size ?p ?pSize)

         (right ?rFrom ?p)
         (left ?rFrom ?p)
         (level ?rFrom ?pSize)

         (left ?rTo ?pNewSucc)

         (level ?rTo ?rToLevel)
         (fit-n-sum ?rToLevel ?pSize ?rToLevelNew)
      )
      :effect (and
         (not (right ?rFrom ?p))
         (not (left ?rFrom ?p))
         (not (left ?rTo ?pNewSucc))
         (rack-succ ?p ?pNewSucc)
         (left ?rTo ?p)

         (not (level ?rFrom ?pSize))
         (not (level ?rTo ?rToLevel))
         (level ?rFrom n00)
         (level ?rTo ?rToLevelNew)
         (increase (total-cost) 1)
      )
   )
)
