(define (domain search-and-rescue)
 (:requirements :probabilistic-effects :typing :fluents)
 (:types loc)
 (:predicates (at ?loc - loc)
              (connected ?locA - loc ?locB - loc)  ;; static
              (has-victim ?l - loc)
              (unknown ?l - loc)
              (loc-prob-low ?l - loc)   ;; static
              (loc-prob-med ?l - loc)   ;; static
              (loc-prob-high ?l - loc)  ;; static
              (victim-boarded)
              (vehicle-empty)
              (rescued ?l - loc)
 )
 (:functions (time) (fuel-spent) )
 
 (:action move-slow-empty
  :parameters (?from - loc ?to - loc)
  :precondition (and (at ?from) (connected ?from ?to) (vehicle-empty))
  :effect (and
            (increase (time) 4)
            (increase (fuel-spent) 1)
            (at ?to) (not (at ?from)))
 )
 (:action move-normal-empty
  :parameters (?from - loc ?to - loc)
  :precondition (and (at ?from) (connected ?from ?to) (vehicle-empty))
  :effect (and
            (increase (time) 2)
            (increase (fuel-spent) 3)
            (probabilistic 0.95 (and (at ?to) (not (at ?from))))
          )
 )
 (:action move-fast-empty
  :parameters (?from - loc ?to - loc)
  :precondition (and (at ?from) (connected ?from ?to) (vehicle-empty))
  :effect (and
            (increase (time) 1)
            (increase (fuel-spent) 9)
            (probabilistic 0.9 (and (at ?to) (not (at ?from))))
          )
 )
 (:action move-slow-loaded
  :parameters (?from - loc ?to - loc)
  :precondition (and (at ?from) (connected ?from ?to) (victim-boarded))
  :effect (and
            (increase (time) 6)
            (increase (fuel-spent) 2)
            (at ?to) (not (at ?from)))
 )
 (:action move-normal-loaded
  :parameters (?from - loc ?to - loc)
  :precondition (and (at ?from) (connected ?from ?to) (victim-boarded))
  :effect (and
            (increase (time) 3)
            (increase (fuel-spent) 6)
            (probabilistic 0.95 (and (at ?to) (not (at ?from))))
          )
 )
 (:action move-fast-loaded
  :parameters (?from - loc ?to - loc)
  :precondition (and (at ?from) (connected ?from ?to) (victim-boarded))
  :effect (and
            (increase (time) 2)
            (increase (fuel-spent) 18)
            (probabilistic 0.9 (and (at ?to) (not (at ?from))))
          )
 )
 (:action explore-cond-high
  :parameters (?l - loc)
  :precondition (and (at ?l) (unknown ?l) (loc-prob-high ?l))
  :effect (and
            (increase (time) 1)
            (not (unknown ?l))
            (probabilistic 0.2 (has-victim ?l) 0.8 (rescued ?l))
          )
 )
 (:action explore-cond-med
  :parameters (?l - loc)
  :precondition (and (at ?l) (unknown ?l) (loc-prob-med ?l))
  :effect (and
            (increase (time) 1)
            (not (unknown ?l))
            (probabilistic 0.1 (has-victim ?l) 0.9 (rescued ?l))
          )
 )
 (:action explore-cond-low
  :parameters (?l - loc)
  :precondition (and (at ?l) (unknown ?l) (loc-prob-low ?l))
  :effect (and
            (increase (time) 1)
            (not (unknown ?l))
            (probabilistic 0.05 (has-victim ?l) 0.95 (rescued ?l))
          )
 )
 (:action board-victim
  :parameters (?l - loc )
  :precondition (and (at ?l) (has-victim ?l))
  :effect (and
            (increase (time) 1)
            (not (has-victim ?l))
            (rescued ?l)
            (not (vehicle-empty))
            (victim-boarded)
          )
 )
)
